@@ -40,17 +40,21 @@
 
 #include <QVarLengthArray>
 #include <QVector>
+#include <functional>
 
 #include "AudioTester.h"
 #include "ProcessPlugin.h"
+#include "WaitFreeFrameBuffer.h"
 #include "jacktrip_types.h"
-//#include "jacktrip_globals.h"
 
 // Forward declarations
 class JackTrip;
 
 // using namespace JackTripNamespace;
 
+// callback function for audio interface errors
+typedef std::function<void(const std::string& errorText)> AudioErrorCallback;
+
 /** \brief Base Class that provides an interface with audio
  */
 class AudioInterface
@@ -76,7 +80,12 @@ class AudioInterface
         UNDEF   ///< Undefined
     };
 
-    enum warningMessageT { DEVICE_WARN_NONE, DEVICE_WARN_LATENCY };
+    enum warningMessageT {
+        DEVICE_WARN_NONE,
+        DEVICE_WARN_BUFFER_LATENCY,
+        DEVICE_WARN_ASIO_LATENCY,
+        DEVICE_WARN_ALSA_LATENCY
+    };
 
     enum errorMessageT {
         DEVICE_ERR_NONE,
@@ -111,6 +120,7 @@ class AudioInterface
 #endif  // endwhere
         AudioInterface::audioBitResolutionT AudioBitResolution = AudioInterface::BIT16,
         bool processWithNetwork = false, JackTrip* jacktrip = nullptr);
+
     /// \brief The class destructor
     virtual ~AudioInterface();
 
@@ -122,24 +132,56 @@ class AudioInterface
      * method to ensure correct inizialization.
      */
     virtual void setup(bool verbose = true);
+
     /// \brief Tell the audio server that we are ready to roll. The
     /// process-callback will start running. This runs on its own thread.
     /// \return 0 on success, otherwise a non-zero error code
     virtual int startProcess() = 0;
+
     /// \brief Stops the process-callback thread
     /// \return 0 on success, otherwise a non-zero error code
     virtual int stopProcess() = 0;
-    /** \brief Process callback. Subclass should call this callback after obtaining the
-    in_buffer and out_buffer pointers.
-    * \param in_buffer Array of input audio samplers for each channel. The user
-    * is responsible to check that each channel has n_frames samplers
-    * \param in_buffer Array of output audio samplers for each channel. The user
-    * is responsible to check that each channel has n_frames samplers
-    */
+
+    /** \brief Broadcast callback. Subclass should call this callback after
+     * obtaining the mon_buffer pointer.
+     *
+     * \param in_buffer Array of input audio samplers for each channel. The user
+     * is responsible to check that each channel has n_frames samplers
+     * \param in_buffer Array of output audio samplers for each channel. The user
+     * is responsible to check that each channel has n_frames samplers
+     */
     virtual void broadcastCallback(QVarLengthArray<sample_t*>& mon_buffer,
                                    unsigned int n_frames);
+
+    /** \brief Audio interface callback. Subclass should call this callback after
+     * obtaining the in_buffer and out_buffer pointers (for duplux mode).
+     *
+     * \param in_buffer Array of input audio samplers for each channel. The user
+     * is responsible to check that each channel has n_frames samplers
+     * \param out_buffer Array of output audio samplers for each channel. The user
+     * is responsible to check that each channel has n_frames samplers
+     */
     virtual void callback(QVarLengthArray<sample_t*>& in_buffer,
                           QVarLengthArray<sample_t*>& out_buffer, unsigned int n_frames);
+
+    /** \brief Audio input process callback. Subclass should call this callback
+     * after obtaining the in_buffer pointer (for input only).
+     *
+     * \param in_buffer Array of input audio samplers for each channel. The user
+     * is responsible to check that each channel has n_frames samplers
+     */
+    virtual void audioInputCallback(QVarLengthArray<sample_t*>& in_buffer,
+                                    unsigned int n_frames);
+
+    /** \brief Audio output process callback. Subclass should call this callback
+     * after obtaining the out_buffer pointer (for output only).
+     *
+     * \param out_buffer Array of output audio samplers for each channel. The user
+     * is responsible to check that each channel has n_frames samplers
+     */
+    virtual void audioOutputCallback(QVarLengthArray<sample_t*>& out_buffer,
+                                     unsigned int n_frames);
+
     /** \brief appendProcessPluginToNetwork(): Append a ProcessPlugin for outgoing audio.
      * The processing order equals order they were appended.
      * This processing is in the JackTrip client before sending to the network.
@@ -148,6 +190,7 @@ class AudioInterface
      * <tt>std::tr1::shared_ptr<ProcessPluginName> loopback(new ProcessPluginName);</tt>
      */
     virtual void appendProcessPluginToNetwork(ProcessPlugin* plugin);
+
     /** \brief appendProcessPluginFromNetwork():
      * Same as appendProcessPluginToNetwork() except that these plugins operate
      * on the audio received from the network (typically from a JackTrip server).
@@ -157,16 +200,20 @@ class AudioInterface
      *               -> JackTrip client -> processPlugin from network -> JACK -> audio
      */
     virtual void appendProcessPluginFromNetwork(ProcessPlugin* plugin);
+
+    /** \brief appendProcessPluginToMonitor():
+     * Appends plugins used for local monitoring
+     */
+    virtual void appendProcessPluginToMonitor(ProcessPlugin* plugin);
+
     /** \brief initPlugins():
      * Initialize all ProcessPlugin modules.
      * The audio sampling rate (mSampleRate) must be set at this time.
      */
-    virtual void appendProcessPluginToMonitor(ProcessPlugin* plugin);
-    /** \brief appendProcessPluginFromNetwork():
-     * Appends plugins used for local monitoring
-     */
     void initPlugins(bool verbose = true);
+
     virtual void connectDefaultPorts() = 0;
+
     /** \brief Convert a 32bit number (sample_t) into one of the bit resolution
      * supported (audioBitResolutionT).
      *
@@ -177,6 +224,7 @@ class AudioInterface
     static void fromSampleToBitConversion(
         const sample_t* const input, int8_t* output,
         const AudioInterface::audioBitResolutionT targetBitResolution);
+
     /** \brief Convert a audioBitResolutionT bit resolution number into a
      * 32bit number (sample_t)
      *
@@ -188,16 +236,17 @@ class AudioInterface
         const int8_t* const input, sample_t* output,
         const AudioInterface::audioBitResolutionT sourceBitResolution);
 
+    /** \brief Sets PIPEWIRE_LATENCY environment variable on unix */
+    static void setPipewireLatency(unsigned int bufferSize, unsigned int sampleRate);
+
     //--------------SETTERS---------------------------------------------
     virtual void setInputChannels(QVarLengthArray<int> inputChans)
     {
         mInputChans = inputChans;
-        mNumInChans = inputChans.size();
     }
     virtual void setOutputChannels(QVarLengthArray<int> outputChans)
     {
         mOutputChans = outputChans;
-        mNumOutChans = outputChans.size();
     }
     virtual void setInputMixMode(inputMixModeT mode) { mInputMixMode = mode; }
     virtual void setSampleRate(uint32_t sample_rate) { mSampleRate = sample_rate; }
@@ -220,6 +269,7 @@ class AudioInterface
     virtual void setLoopBack(bool b) { mLoopBack = b; }
     virtual void enableBroadcastOutput() {}
     virtual void setAudioTesterP(AudioTester* atp) { mAudioTesterP = atp; }
+    void setErrorCallback(AudioErrorCallback c) { mErrorCallback = c; }
     //------------------------------------------------------------------
 
     //--------------GETTERS---------------------------------------------
@@ -227,6 +277,8 @@ class AudioInterface
     virtual int getNumInputChannels() const { return mInputChans.size(); }
     /// \brief Get Number of Output Channels
     virtual int getNumOutputChannels() const { return mOutputChans.size(); }
+    /// \brief Get Number of Monitor Channels
+    virtual int getNumMonChannels() const { return mOutputChans.size(); }
     virtual QVarLengthArray<int> getInputChannels() const { return mInputChans; }
     virtual QVarLengthArray<int> getOutputChannels() const { return mOutputChans; }
     virtual inputMixModeT getInputMixMode() const { return mInputMixMode; }
@@ -251,11 +303,12 @@ class AudioInterface
      * \return Sample Rate in Hz
      */
     static int getSampleRateFromType(samplingRateT rate_type);
-    std::string getDevicesWarningMsg();
-    std::string getDevicesErrorMsg();
-    std::string getDevicesWarningHelpUrl();
-    std::string getDevicesErrorHelpUrl();
-
+    const std::string& getDevicesWarningMsg() const { return mWarningMsg; }
+    const std::string& getDevicesErrorMsg() const { return mErrorMsg; }
+    const std::string& getDevicesWarningHelpUrl() const { return mWarningHelpUrl; }
+    const std::string& getDevicesErrorHelpUrl() const { return mErrorHelpUrl; }
+    bool highLatencyBufferSize() const { return getBufferSizeInSamples() > 256; }
+    bool getHighLatencyFlag() const { return mHighLatencyFlag; }
     //------------------------------------------------------------------
 
    private:
@@ -273,10 +326,8 @@ class AudioInterface
     QVarLengthArray<sample_t*>
         mNetInBuffer;  ///< Vector of Input buffers/channel read from net
     QVarLengthArray<sample_t*>
-        mAPInBuffer;  ///< Vector of Input buffers/channel for AllPass input
-#endif                // endwhere
-    QVarLengthArray<sample_t*>
-        mInBufCopy;           ///< needed in callback() to modify JACK audio input
+        mAPInBuffer;          ///< Vector of Input buffers/channel for AllPass input
+#endif                        // endwhere
     int mAudioBitResolution;  ///< Bit resolution in audio samples
     AudioInterface::audioBitResolutionT
         mBitResolutionMode;  ///< Bit resolution (audioBitResolutionT) mode
@@ -295,20 +346,19 @@ class AudioInterface
         mInProcessBuffer;  ///< Vector of Input buffers/channel for ProcessPlugin
     QVarLengthArray<sample_t*>
         mOutProcessBuffer;  ///< Vector of Output buffers/channel for ProcessPlugin
-    QVarLengthArray<sample_t*>
-        mMonProcessBuffer;       ///< Vector of Monitor buffers/channel for ProcessPlugin
+    WaitFreeFrameBuffer<64>*
+        mMonitorQueuePtr;        //< Queue of audio frames from monitor signal
     int8_t* mAudioInputPacket;   ///< Packet containing all the channels to read from the
                                  ///< RingBuffer
     int8_t* mAudioOutputPacket;  ///< Packet containing all the channels to send to the
                                  ///< RingBuffer
     bool mLoopBack;
     bool mProcessWithNetwork;  ///< whether or not to send/receive data via the network
+    bool mMonitorStarted;      ///< True if we have started to consume monitor audio
     AudioTester* mAudioTesterP{nullptr};
 
    protected:
     JackTrip* mJackTrip;          ///< JackTrip Mediator Class pointer
-    int mNumInChans;              ///< Number of Input Channels
-    int mNumOutChans;             ///<  Number of Output Channels
     inputMixModeT mInputMixMode;  ///< Input mixing mode
 
     void setDevicesWarningMsg(warningMessageT msg);
@@ -321,6 +371,8 @@ class AudioInterface
     std::string mErrorMsg;
     std::string mWarningHelpUrl;
     std::string mErrorHelpUrl;
+    bool mHighLatencyFlag;
+    AudioErrorCallback mErrorCallback;
 };
 
 #endif  // __AUDIOINTERFACE_H__
