@@ -88,6 +88,12 @@ void JackAudioInterface::setup(bool verbose)
     setupClient();
     AudioInterface::setup(verbose);
     setProcessCallback();
+    if (highLatencyBufferSize()) {
+        AudioInterface::setDevicesWarningMsg(AudioInterface::DEVICE_WARN_BUFFER_LATENCY);
+    } else {
+        AudioInterface::setDevicesWarningMsg(AudioInterface::DEVICE_WARN_NONE);
+    }
+    AudioInterface::setDevicesErrorMsg(AudioInterface::DEVICE_ERR_NONE);
 }
 
 //*******************************************************************************
@@ -117,6 +123,8 @@ void JackAudioInterface::setupClient()
     /// verbose message, check how to desable them.
     {
         QMutexLocker locker(&sJackMutex);
+        // TODO: this needs a timeout because it will hang indefinitely
+        // if the Jack server is not running
 //#ifndef WAIR // WAIR
 //        mClient = jack_client_open (client_name, options, &status, server_name);
 //#else
@@ -152,7 +160,7 @@ void JackAudioInterface::setupClient()
     }
 
     // Set function to call if Jack shuts down
-    jack_on_info_shutdown(mClient, this->jackShutdown, 0);
+    jack_on_info_shutdown(mClient, this->jackShutdown, this);
 
     // Create input and output channels
     createChannels();
@@ -161,17 +169,17 @@ void JackAudioInterface::setupClient()
     mNumFrames = getBufferSizeInSamples();
 
     // Initialize Buffer array to read and write audio
-    mInBuffer.resize(mNumInChans);
-    mOutBuffer.resize(mNumOutChans);
-    mBroadcastBuffer.resize(mNumOutChans);
+    mInBuffer.resize(getNumInputChannels());
+    mOutBuffer.resize(getNumOutputChannels());
+    mBroadcastBuffer.resize(getNumOutputChannels());
 }
 
 //*******************************************************************************
 void JackAudioInterface::createChannels()
 {
     // Create Input Ports
-    mInPorts.resize(mNumInChans);
-    for (int i = 0; i < mNumInChans; i++) {
+    mInPorts.resize(getNumInputChannels());
+    for (int i = 0; i < getNumInputChannels(); i++) {
         QString inName;
         QTextStream(&inName) << "send_" << i + 1;
         mInPorts[i] =
@@ -180,8 +188,8 @@ void JackAudioInterface::createChannels()
     }
 
     // Create Output Ports
-    mOutPorts.resize(mNumOutChans);
-    for (int i = 0; i < mNumOutChans; i++) {
+    mOutPorts.resize(getNumOutputChannels());
+    for (int i = 0; i < getNumOutputChannels(); i++) {
         QString outName;
         QTextStream(&outName) << "receive_" << i + 1;
         mOutPorts[i] =
@@ -190,8 +198,8 @@ void JackAudioInterface::createChannels()
     }
     // Create Broadcast Ports
     if (mBroadcast) {
-        mBroadcastPorts.resize(mNumOutChans);
-        for (int i = 0; i < mNumOutChans; i++) {
+        mBroadcastPorts.resize(getNumOutputChannels());
+        for (int i = 0; i < getNumOutputChannels(); i++) {
             QString outName;
             QTextStream(&outName) << "broadcast_" << i + 1;
             mBroadcastPorts[i] =
@@ -267,11 +275,22 @@ int JackAudioInterface::stopProcess()
 
 //*******************************************************************************
 void JackAudioInterface::jackShutdown(jack_status_t /*code*/, const char* reason,
-                                      void* /*arg*/)
+                                      void* arg)
 {
-    std::cout << reason << std::endl;
-    JackTrip::sJackStopped = true;
-    std::cout << "The Jack Server was shut down!" << std::endl;
+    std::string errorMsg = "The Jack server was shut down";
+    if (reason != nullptr) {
+        errorMsg += ": ";
+        errorMsg += reason;
+    }
+    if (arg != nullptr) {
+        JackAudioInterface* ifPtr = static_cast<JackAudioInterface*>(arg);
+        ifPtr->mErrorMsg          = errorMsg;
+        if (ifPtr->mErrorCallback) {
+            ifPtr->mErrorCallback(errorMsg);
+        }
+    }
+    std::cerr << errorMsg << std::endl;
+    JackTrip::sAudioStopped = true;
 }
 
 //*******************************************************************************
@@ -285,12 +304,12 @@ int JackAudioInterface::processCallback(jack_nframes_t nframes)
 
     // Get input and output buffers from JACK
     //-------------------------------------------------------------------
-    for (int i = 0; i < mNumInChans; i++) {
+    for (int i = 0; i < getNumInputChannels(); i++) {
         // Input Ports are READ ONLY and change as needed (no locks) - make a copy for
         // debugging
         mInBuffer[i] = (sample_t*)jack_port_get_buffer(mInPorts[i], nframes);
     }
-    for (int i = 0; i < mNumOutChans; i++) {
+    for (int i = 0; i < getNumOutputChannels(); i++) {
         // Output Ports are WRITABLE
         mOutBuffer[i] = (sample_t*)jack_port_get_buffer(mOutPorts[i], nframes);
     }
@@ -305,7 +324,7 @@ int JackAudioInterface::processCallback(jack_nframes_t nframes)
     AudioInterface::callback(mInBuffer, mOutBuffer, nframes);
 
     if (mBroadcast) {
-        for (int i = 0; i < mNumOutChans; i++) {
+        for (int i = 0; i < getNumOutputChannels(); i++) {
             // Broadcast Ports are WRITABLE
             mBroadcastBuffer[i] =
                 (sample_t*)jack_port_get_buffer(mBroadcastPorts[i], nframes);
@@ -333,7 +352,7 @@ void JackAudioInterface::connectDefaultPorts()
         cout << "WARNING: Cannot find any physical capture ports" << endl;
     } else {
         // Connect capure ports to jacktrip send
-        for (int i = 0; i < mNumInChans; i++) {
+        for (int i = 0; i < getNumInputChannels(); i++) {
             // Check that we don't run out of capture ports
             if (ports[i] != NULL) {
                 jack_connect(mClient, ports[i], jack_port_name(mInPorts[i]));
@@ -341,7 +360,7 @@ void JackAudioInterface::connectDefaultPorts()
                 break;
             }
         }
-        std::free(ports);
+        jack_free(ports);
     }
 
     // Get physical input (playback) ports
@@ -351,7 +370,7 @@ void JackAudioInterface::connectDefaultPorts()
         cout << "WARNING: Cannot find any physical playback ports" << endl;
     } else {
         // Connect playback ports to jacktrip receive
-        for (int i = 0; i < mNumOutChans; i++) {
+        for (int i = 0; i < getNumOutputChannels(); i++) {
             // Check that we don't run out of capture ports
             if (ports[i] != NULL) {
                 jack_connect(mClient, jack_port_name(mOutPorts[i]), ports[i]);
@@ -359,6 +378,6 @@ void JackAudioInterface::connectDefaultPorts()
                 break;
             }
         }
-        std::free(ports);
+        jack_free(ports);
     }
 }
