@@ -96,8 +96,8 @@ class StdDev
 {
    public:
     StdDev(int id, QElapsedTimer* timer, int w);
-    void tick();
-    double calcAuto(double autoHeadroom, double localFPPdur, double peerFPPdur);
+    bool tick();  // returns true if stats were updated
+    double calcAuto();
     int mId;
     int plcOverruns;
     int plcUnderruns;
@@ -113,8 +113,10 @@ class StdDev
     double longTermStdDevAcc;
     double longTermMax;
     double longTermMaxAcc;
+    int longTermCnt;
 
    private:
+    double smooth(double avg, double current);
     void reset();
     QElapsedTimer* mTimer;
     std::vector<double> data;
@@ -124,7 +126,6 @@ class StdDev
     double min;
     double max;
     int ctr;
-    int longTermCnt;
 };
 
 class Regulator : public RingBuffer
@@ -185,11 +186,14 @@ class Regulator : public RingBuffer
    private:
     void shimFPP(const int8_t* buf, int len, int seq_num);
     void pushPacket(const int8_t* buf, int seq_num);
+    void assemblePacket(const int8_t* buf, int peer_seq_num);
     void pullPacket();
+    void updateTolerance();
     void setFPPratio();
-    bool mFPPratioIsSet;
     void processPacket(bool glitch);
     void processChannel(int ch, bool glitch, int packetCnt, bool lastWasGlitch);
+
+    bool mFPPratioIsSet;
     int mNumChannels;
     int mAudioBitRes;
     int mFPP;
@@ -221,15 +225,14 @@ class Regulator : public RingBuffer
     int mLastSeqNumOut;
     std::vector<double> mPhasor;
     std::vector<double> mIncomingTiming;
-    int mModSeqNum;
-    int mLostWindow;
+    std::vector<int> mAssemblyCounts;
     int mSkip;
     int mFPPratioNumerator;
     int mFPPratioDenominator;
-    int mAssemblyCnt;
-    int mModCycle;
     bool mAuto;
-    int mModSeqNumPeer;
+    bool mSkipAutoHeadroom;
+    int mLastGlitches;
+    double mCurrentHeadroom;
     double mAutoHeadroom;
     double mFPPdurMsec;
     double mPeerFPPdurMsec;
@@ -262,6 +265,7 @@ class RegulatorWorker : public QObject
         , mPacketQueue(rPtr->getPacketSize())
         , mPacketQueueTarget(1)
         , mLastUnderrun(0)
+        , mSkipQueueUpdate(true)
         , mUnderrun(false)
         , mStarted(false)
     {
@@ -312,12 +316,20 @@ class RegulatorWorker : public QObject
     {
         if (mUnderrun.load(std::memory_order_relaxed)) {
             if (mStarted) {
-                // allow up to 1 underrun per second before adjusting target
                 double now =
                     (double)mRegulatorPtr->mIncomingTimer.nsecsElapsed() / 1000000.0;
-                if (mLastUnderrun != 0 && now - mLastUnderrun < 1000.0)
-                    updateQueueTarget();
-                mLastUnderrun = now;
+                // only adjust target at most once per 1.0 seconds
+                if (now - mLastUnderrun >= 1000.0) {
+                    if (mSkipQueueUpdate) {
+                        // require consecutive underruns periods to bump target
+                        mSkipQueueUpdate = false;
+                    } else if (now - mLastUnderrun < 2000.0) {
+                        // previous period had underruns
+                        updateQueueTarget();
+                        mSkipQueueUpdate = true;
+                    }  // else, skip this period but not the next one
+                    mLastUnderrun = now;
+                }
                 mUnderrun.store(false, std::memory_order_relaxed);
             } else {
                 mStarted = true;
@@ -334,24 +346,19 @@ class RegulatorWorker : public QObject
    private:
     void updateQueueTarget()
     {
-        // cap queue size at 4x the time it takes to run a prediction
-        double samples =
-            (mRegulatorPtr->getLastDspElapsed() * 4 * mRegulatorPtr->getSampleRate())
-            / 1000;
-        std::size_t maxPackets = (samples / mRegulatorPtr->getBufferSizeInSamples()) + 1;
-        if (maxPackets > mPacketQueue.capacity() / 2)
-            maxPackets = mPacketQueue.capacity() / 2;
-        if (mPacketQueueTarget < maxPackets) {
-            // adjust queue target
-            ++mPacketQueueTarget;
-            std::cout << "PLC worker queue: adjusting target=" << mPacketQueueTarget
-                      << " (max=" << maxPackets
-                      << ", lastDspElapsed=" << mRegulatorPtr->getLastDspElapsed() << ")"
-                      << std::endl;
-            if (mPacketQueueTarget == maxPackets) {
-                emit signalMaxQueueSize();
-                std::cout << "PLC worker queue: reached MAX target!" << std::endl;
-            }
+        // sanity check
+        const std::size_t maxPackets = mPacketQueue.capacity() / 2;
+        if (mPacketQueueTarget > maxPackets)
+            return;
+        // adjust queue target
+        ++mPacketQueueTarget;
+        std::cout << "PLC worker queue: adjusting target=" << mPacketQueueTarget
+                  << " (max=" << maxPackets
+                  << ", lastDspElapsed=" << mRegulatorPtr->getLastDspElapsed() << ")"
+                  << std::endl;
+        if (mPacketQueueTarget == maxPackets) {
+            emit signalMaxQueueSize();
+            std::cout << "PLC worker queue: reached MAX target!" << std::endl;
         }
     }
 
@@ -367,6 +374,9 @@ class RegulatorWorker : public QObject
     /// time of last underrun, in milliseconds
     double mLastUnderrun;
 
+    /// true if the next packet queue update should be skipped
+    bool mSkipQueueUpdate;
+
     /// last value of packet queue underruns
     std::atomic<bool> mUnderrun;
 
