@@ -91,8 +91,7 @@ UdpDataProtocol::UdpDataProtocol(JackTrip* jacktrip, const runModeT runmode,
     , mControlPacketSize(63)
     , mStopSignalSent(false)
 {
-    mStopped = false;
-    mIPv6    = false;
+    mIPv6 = false;
     std::memset(&mPeerAddr, 0, sizeof(mPeerAddr));
     std::memset(&mPeerAddr6, 0, sizeof(mPeerAddr6));
     mPeerAddr.sin_port   = htons(mPeerPort);
@@ -338,7 +337,7 @@ void UdpDataProtocol::processControlPacket(const char* buf)
 //*******************************************************************************
 int UdpDataProtocol::receivePacket(char* buf, const size_t n)
 {
-    int n_bytes = ::recv(mSocket, buf, n, 0);
+    int n_bytes = ::recv(mSocket, buf, int(n), 0);
     if (n_bytes == mControlPacketSize) {
         processControlPacket(buf);
         return 0;
@@ -366,11 +365,11 @@ functions. DWORD n_bytes; WSABUF buffer; int error; buffer.len = n; buffer.buf =
 #else*/
     int n_bytes;
     if (mIPv6) {
-        n_bytes = ::sendto(mSocket, buf, n, 0, (struct sockaddr*)&mPeerAddr6,
+        n_bytes = ::sendto(mSocket, buf, int(n), 0, (struct sockaddr*)&mPeerAddr6,
                            sizeof(mPeerAddr6));
     } else {
-        n_bytes =
-            ::sendto(mSocket, buf, n, 0, (struct sockaddr*)&mPeerAddr, sizeof(mPeerAddr));
+        n_bytes = ::sendto(mSocket, buf, int(n), 0, (struct sockaddr*)&mPeerAddr,
+                           sizeof(mPeerAddr));
     }
     return n_bytes;
     //#endif
@@ -551,6 +550,8 @@ void UdpDataProtocol::run()
     //
     // clang-format off
 
+    threadHasStarted();
+
     switch (mRunMode) {
     case RECEIVER: {
         // Connect signals and slots for packets arriving too late notifications
@@ -671,7 +672,7 @@ void UdpDataProtocol::run()
             // This QT method gave me a lot of trouble, so I replaced it with my own 'waitForReady'
             // that uses signals and slots and can also report with packets have not
             // arrive for a longer time
-            //timeout = UdpSocket.waitForReadyRead(30);
+            //timeout = UdpSocket.waitForReadyRead(gUdpWaitTimeout);
             //        timeout = cc unused!
 #if defined (MANUAL_POLL)
             waitForReady(60000); //60 seconds
@@ -749,14 +750,14 @@ void UdpDataProtocol::run()
         full_redundant_packet = new int8_t[full_redundant_packet_size];
         std::memset(full_redundant_packet, 0,
                     full_redundant_packet_size); // Initialize to 0
-        while (!mStopped && !JackTrip::sSigInt && !JackTrip::sJackStopped) {
+        while (!mStopped && !JackTrip::sSigInt && !JackTrip::sAudioStopped) {
             sendPacketRedundancy(full_redundant_packet, full_redundant_packet_size,
                                  full_packet_size);
         }
 
         // Send exit packet (with 1 redundant packet).
         cout << "sending exit packet" << endl;
-        QByteArray exitPacket = QByteArray(mControlPacketSize, 0xff);
+        QByteArray exitPacket = QByteArray(mControlPacketSize, static_cast<char>(0xff));
         sendPacket(exitPacket.constData(), mControlPacketSize);
         sendPacket(exitPacket.constData(), mControlPacketSize);
         emit signalCeaseTransmission();
@@ -799,10 +800,12 @@ void UdpDataProtocol::waitForReady(int timeout_msec)
 //*******************************************************************************
 void UdpDataProtocol::printUdpWaitedTooLong(int wait_msec)
 {
-    int wait_time = 30;  // msec
-    if (!(wait_msec % wait_time)) {
-        std::cerr << "UDP waiting too long (more than " << wait_time << "ms) for "
-                  << mPeerAddress.toString().toStdString() << "..." << endl;
+    if (!(wait_msec % gUdpWaitTimeout)) {
+        // only log error once per gap in audio, rather than every 30ms
+        if (wait_msec <= gUdpWaitTimeout) {
+            std::cerr << "UDP waiting too long (more than " << gUdpWaitTimeout << "ms) for "
+                    << mPeerAddress.toString().toStdString() << "..." << endl;
+        }
         emit signalUdpWaitingTooLong();
     }
 }
@@ -941,7 +944,7 @@ void UdpDataProtocol::sendPacketRedundancy(int8_t* full_redundant_packet,
     int8_t* src = mAudioPacket;
     if (1 < mChans) {
         // Convert internal interleaved layout to non-interleaved
-        int N       = getAudioPacketSizeInBites() / mChans / mSmplSize;
+        int N       = int(getAudioPacketSizeInBites() / mChans / mSmplSize);
         int8_t* dst = mBuffer.data();
         for (int n = 0; n < N; ++n) {
             for (int c = 0; c < mChans; ++c) {
