@@ -80,6 +80,7 @@ UdpHubListener::UdpHubListener(int server_port, int server_udp_port, QObject* pa
            "client fan out/in, including server", "full mix, including server"})
     , m_connectDefaultAudioPorts(false)
     , mIOStatTimeout(0)
+    , mRegulatorThreadPtr(NULL)
 {
     // Register JackTripWorker with the hub listener
     // mJTWorker = new JackTripWorker(this);
@@ -121,7 +122,13 @@ UdpHubListener::UdpHubListener(int server_port, int server_udp_port, QObject* pa
 //*******************************************************************************
 UdpHubListener::~UdpHubListener()
 {
+    mStopCheckTimer.stop();
     QMutexLocker lock(&mMutex);
+    if (mRegulatorThreadPtr != NULL) {
+        mRegulatorThreadPtr->quit();
+        mRegulatorThreadPtr->wait();
+        delete mRegulatorThreadPtr;
+    }
     // delete mJTWorker;
     for (int i = 0; i < gMaxThreads; i++) {
         delete mJTWorkers->at(i);
@@ -218,7 +225,7 @@ void UdpHubListener::start()
             emit signalError(error_message);
             return;
         }
-        mAuth.reset(new Auth(mCredsFile));
+        mAuth.reset(new Auth(mCredsFile, true));
     }
 
     cout << "JackTrip HUB SERVER: Waiting for client connections..." << endl;
@@ -230,6 +237,18 @@ void UdpHubListener::start()
     mStopCheckTimer.setInterval(200);
     connect(&mStopCheckTimer, &QTimer::timeout, this, &UdpHubListener::stopCheck);
     mStopCheckTimer.start();
+
+#ifdef REGULATOR_SHARED_WORKER_THREAD
+    // Start regulator thread if bufstrategy == 3
+    if (mBufferStrategy == 3) {
+        // create shared regulator thread
+        mRegulatorThreadPtr = new QThread();
+        mRegulatorThreadPtr->setObjectName("RegulatorThread");
+        mRegulatorThreadPtr->start();
+    }
+#endif
+
+    emit signalStarted();
 }
 
 void UdpHubListener::receivedNewConnection()
@@ -345,6 +364,7 @@ void UdpHubListener::receivedClientInfo(QSslSocket* clientConnection)
         mJTWorkers->at(id)->setIOStatStream(mIOStatStream);
     }
     mJTWorkers->at(id)->setBufferStrategy(mBufferStrategy);
+    mJTWorkers->at(id)->setRegulatorThread(mRegulatorThreadPtr);
     mJTWorkers->at(id)->setNetIssuesSimulation(mSimulatedLossRate, mSimulatedJitterRate,
                                                mSimulatedDelayRel);
     mJTWorkers->at(id)->setBroadcast(mBroadcastQueue);
@@ -668,6 +688,11 @@ void UdpHubListener::stopAllThreads()
             iterator.next();
         }
     }
+    if (mRegulatorThreadPtr != nullptr) {
+        // Stop the Regulator thread
+        mRegulatorThreadPtr->quit();
+        mRegulatorThreadPtr->wait();
+    }
 }
 // TODO:
 // USE bool QAbstractSocket::isValid () const to check if socket is connect. if not, exit
