@@ -44,6 +44,7 @@
 #endif
 
 #ifdef RT_AUDIO
+#include "../RtAudioInterface.h"
 #include "RtAudio.h"
 #endif
 
@@ -929,11 +930,11 @@ void QJackTrip::start()
 
 #ifdef RT_AUDIO
             if (m_ui->backendComboBox->currentIndex() == 1) {
+                unsigned int bufferSize = m_ui->bufferSizeComboBox->currentText().toInt();
+                unsigned int sampleRate = m_ui->sampleRateComboBox->currentText().toInt();
                 m_jackTrip->setAudiointerfaceMode(JackTrip::RTAUDIO);
-                m_jackTrip->setSampleRate(
-                    m_ui->sampleRateComboBox->currentText().toInt());
-                m_jackTrip->setAudioBufferSizeInSamples(
-                    m_ui->bufferSizeComboBox->currentText().toInt());
+                m_jackTrip->setSampleRate(sampleRate);
+                m_jackTrip->setAudioBufferSizeInSamples(bufferSize);
                 // we assume that first entry is "(default)"
                 if (m_ui->inputDeviceComboBox->currentIndex() == 0) {
                     m_jackTrip->setInputDevice("");
@@ -947,6 +948,7 @@ void QJackTrip::start()
                     m_jackTrip->setOutputDevice(
                         m_ui->outputDeviceComboBox->currentText().toStdString());
                 }
+                AudioInterface::setPipewireLatency(bufferSize, sampleRate);
             }
 #endif
 
@@ -1123,6 +1125,7 @@ void QJackTrip::virtualStudioMode()
 {
     this->hide();
     m_vs->show();
+    m_vs->toVirtualStudio();
 }
 #endif
 
@@ -1984,26 +1987,21 @@ void QJackTrip::populateDeviceMenu(QComboBox* menu, bool isInput)
     menu->clear();
     menu->addItem(QStringLiteral("(default)"));
 
-    std::vector<RtAudio::Api> apis;
-    RtAudio::getCompiledApi(apis);
+    QVector<RtAudioDevice> devices;
+    RtAudioInterface::scanDevices(devices);
+    for (auto info : devices) {
+        // convert names to QString to gracefully handle invalid
+        // utf8 character sequences, such as "RØDE Microphone"
+        const QString utf8Name(QString::fromStdString(info.name));
 
-    for (uint32_t i = 0; i < apis.size(); i++) {
-        RtAudio rtaudio(apis.at(i));
-        unsigned int devices = rtaudio.getDeviceCount();
-        for (unsigned int j = 0; j < devices; j++) {
-            RtAudio::DeviceInfo info = rtaudio.getDeviceInfo(j);
-            if (info.probed == true) {
-                // Don't include duplicate entries
-                if (menu->findText(QString::fromStdString(info.name)) != -1) {
-                    continue;
-                }
-
-                if (isInput && info.inputChannels > 0) {
-                    menu->addItem(QString::fromStdString(info.name));
-                } else if (!isInput && info.outputChannels > 0) {
-                    menu->addItem(QString::fromStdString(info.name));
-                }
-            }
+        // Don't include duplicate entries
+        if (menu->findText(utf8Name) != -1) {
+            continue;
+        }
+        if (isInput && info.inputChannels > 0) {
+            menu->addItem(utf8Name);
+        } else if (!isInput && info.outputChannels > 0) {
+            menu->addItem(utf8Name);
         }
     }
 
